@@ -1,19 +1,20 @@
 /*
- *  This file is a part of Libint.
- *  Copyright (C) 2004-2014 Edward F. Valeev
+ *  Copyright (C) 2004-2019 Edward F. Valeev
  *
- *  This program is free software: you can redistribute it and/or modify
+ *  This file is part of Libint.
+ *
+ *  Libint is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Libint is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see http://www.gnu.org/licenses/.
+ *  along with Libint.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -30,12 +31,18 @@
 
 #include <iostream>
 #include <fstream>
+#include <limits>
 #include <vector>
 #include <boost/preprocessor.hpp>
+#if not BOOST_PP_VARIADICS  // no variadic macros? your compiler is out of date! (should not be possible since variadic macros are part of C++11)
+#  error "your compiler does not provide variadic macros (but does support C++11), something is seriously broken, please create an issue at https://github.com/evaleev/libint/issues"
+#endif
+
 
 #include <libint2/config.h>
 #include <global_macros.h>
 #include <libint2/cgshell_ordering.h>
+#include <libint2/shgshell_ordering.h>
 
 #include <default_params.h>
 #include <rr.h>
@@ -51,7 +58,7 @@
 #include <dims.h>
 #include <purgeable.h>
 #include <buildtest.h>
-#include <deriv_iter.h>
+#include <libint2/deriv_iter.h>
 
 #include <master_ints_list.h>
 
@@ -237,16 +244,21 @@ namespace {
     typedef EmptySet type;
   };
 
-  template <typename OperDescrType> OperDescrType make_descr(int, int, int) {
+  template <typename OperDescrType> OperDescrType make_descr(int, int, int = 0) {
     return OperDescrType();
   }
   template <> CartesianMultipole_Descr<3u> make_descr<CartesianMultipole_Descr<3u>>(int x, int y, int z) {
     CartesianMultipole_Descr<3u> result;
+    // cartesian multipole quanta
     result.inc(0,x);
     result.inc(1,y);
     result.inc(2,z);
     return result;
   }
+  template <> SphericalMultipole_Descr make_descr<SphericalMultipole_Descr>(int l, int m, int) {
+    SphericalMultipole_Descr result(l,m);
+    return result;
+  }
 }
 
 template <typename _OperType>
@@ -257,7 +269,6 @@ build_onebody_1b_1k(std::ostream& os, std::string label, const SafePtr<Compilati
   // implement overlap as a special case of cartesian emultipole
   using OperType = typename std::conditional<std::is_same<_OperType,OverlapOper>::value,CartesianMultipoleOper<3u>,_OperType>::type;
   const std::string task = task_label(label, deriv_level);
-  const std::string task_uc = task_label(label, deriv_level);
   typedef CGShell BFType;
   typedef typename OperType::Descriptor OperDescrType;
   typedef GenIntegralSet_1_1<CGShell, OperType, typename AuxQuantaType<OperType>::type> Onebody_sh_1_1;
@@ -271,10 +282,25 @@ build_onebody_1b_1k(std::ostream& os, std::string label, const SafePtr<Compilati
 
   LibraryTaskManager& taskmgr = LibraryTaskManager::Instance();
   taskmgr.current(task);
-  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task_uc,lmax));
+  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task,lmax));
 
   const auto nullaux = typename Onebody_sh_1_1::AuxIndexType(0u);
 
+  // optionally skip derivative property ints
+#ifdef DISABLE_ONEBODY_PROPERTY_DERIVS
+  const auto property_operator = !(std::is_same<_OperType,OverlapOper>::value ||
+                                   std::is_same<_OperType,KineticOper>::value ||
+                                   std::is_same<_OperType,ElecPotOper>::value);
+  if (property_operator && deriv_level > 0)
+    return;
+#endif
+  // derivatives of spherical multipole integrals are not implemented
+  {
+    if (std::is_same<_OperType,SphericalMultipoleOper>::value && deriv_level > 0)
+      throw std::invalid_argument("derivatives of spherical multipole ints are not yet implemented");
+  }
+
+
   //
   // Construct graphs for each desired target integral and
   // 1) generate source code for the found traversal path
@@ -286,7 +312,6 @@ build_onebody_1b_1k(std::ostream& os, std::string label, const SafePtr<Compilati
   //
   SafePtr<DirectedGraph> dg(new DirectedGraph);
   SafePtr<Strategy> strat(new Strategy());
-  SafePtr<Tactic> tactic(new FirstChoiceTactic<DummyRandomizePolicy>);
   SafePtr<CodeContext> context(new CppCodeContext(cparams));
   SafePtr<MemoryManager> memman(new WorstFitMemoryManager());
 
@@ -300,18 +325,7 @@ build_onebody_1b_1k(std::ostream& os, std::string label, const SafePtr<Compilati
              )
             continue;
 
-          // unroll only if max_am <= cparams->max_am_opt(task)
-          using std::max;
-          const unsigned int max_am = max(la,lb);
-          const bool need_to_optimize = (max_am <= cparams->max_am_opt(task));
-          const bool need_to_unroll = l_to_cgshellsize(la)*l_to_cgshellsize(lb) <= cparams->unroll_threshold();
-          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? 1000000000 : 1;
-
-          dg->registry()->unroll_threshold(unroll_threshold);
-          dg->registry()->do_cse(need_to_optimize);
-          dg->registry()->condense_expr(condense_expr(cparams->unroll_threshold(),cparams->max_vector_length()>1));
-          // Need to accumulate integrals?
-          dg->registry()->accumulate_targets(cparams->accumulate_targets());
+          SafePtr<Tactic> tactic(new TwoCenter_OS_Tactic(la,lb));
 
           // this will hold all target shell sets
           std::vector< SafePtr<Onebody_sh_1_1> > targets;
@@ -345,6 +359,18 @@ build_onebody_1b_1k(std::ostream& os, std::string label, const SafePtr<Compilati
               END_FOR_CART
             }
           }
+          if (std::is_same<_OperType,SphericalMultipoleOper>::value) {
+            // reset descriptors array
+            descrs.resize(0);
+            // iterate over operators and construct their descriptors
+            for(int l=0; l<=MULTIPOLE_MAX_ORDER; ++l) {
+              // we iterate over them same way as over solid harmonic Gaussian shells
+              int m;
+              FOR_SOLIDHARM(l,m)
+                descrs.push_back(make_descr<OperDescrType>(l,m));
+              END_FOR_SOLIDHARM
+            }
+          }
 
           // derivative index is the outermost (slowest running)
           // operator component is second slowest
@@ -352,30 +378,19 @@ build_onebody_1b_1k(std::ostream& os, std::string label, const SafePtr<Compilati
           ////////////
           // loop over unique derivative index combinations
           ////////////
-          // skip 1 center -- all derivatives with respect to that center can be
-          // recovered using translational invariance conditions
-          // which center to skip? (A = 0, B = 1, C = 2)
-          // case1: operator is coordinate-independent (overlap, kinetic) -> B
-          // case2: operator is coordinate-dependent (elecpot, emultipole) -> C
-          constexpr bool coordinate_independent_oper = std::is_same<_OperType,OverlapOper>::value ||
-                                                       std::is_same<_OperType,KineticOper>::value;
-          constexpr unsigned int center_to_skip = (coordinate_independent_oper) ? 1 : 2;
-          constexpr unsigned int num_deriv_centers = (coordinate_independent_oper) ? 2 : 3;
-          DerivIndexIterator<num_deriv_centers - 1> diter(deriv_level);
+          // NB translational invariance is now handled by CR_DerivGauss
+          CartesianDerivIterator<2> diter(deriv_level);
           bool last_deriv = false;
           do {
             BFType a(la);
             BFType b(lb);
             
-            for(unsigned int c=0, center=0; c<num_deriv_centers; ++c) {
-              if (c == center_to_skip)
-                continue;
+            for(unsigned int c=0; c!=2; ++c) {
               const unsigned int ndir = std::is_same<BFType,CGShell>::value ? 3 : 1;
               for(unsigned int xyz=0; xyz<ndir; ++xyz) {
-                if (c == 0) a.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (c == 1) b.deriv().inc(xyz, diter.value(3 * center + xyz));
+                if (c == 0) a.deriv().inc(xyz, (*diter).at(xyz));
+                if (c == 1) b.deriv().inc(xyz, (*diter).at(3 + xyz));
               }
-              ++center;
             }
 
             // operator component loop
@@ -390,6 +405,21 @@ build_onebody_1b_1k(std::ostream& os, std::string label, const SafePtr<Compilati
             if (!last_deriv) diter.next();
           } while (!last_deriv); // loop over derivatives
 
+          // unroll only if max_am <= cparams->max_am_opt(task)
+          using std::max;
+          const unsigned int max_am = max(la,lb);
+          const auto nopers = descrs.size();
+          const bool need_to_optimize = (max_am <= cparams->max_am_opt(task));
+          // decide whether to unroll based on the aggregate size of undifferentiated quartets
+          const bool need_to_unroll = nopers * l_to_cgshellsize(la)*l_to_cgshellsize(lb) <= cparams->unroll_threshold();
+          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? std::numeric_limits<unsigned int>::max() : 0;
+
+          dg->registry()->unroll_threshold(unroll_threshold);
+          dg->registry()->do_cse(need_to_optimize);
+          dg->registry()->condense_expr(condense_expr(cparams->unroll_threshold(),cparams->max_vector_length()>1));
+          // Need to accumulate integrals?
+          dg->registry()->accumulate_targets(cparams->accumulate_targets());
+
           // shove all targets on the graph, IN ORDER
           for(auto t = targets.begin(); t!=targets.end(); ++t) {
             SafePtr<DGVertex> t_ptr = dynamic_pointer_cast<DGVertex,Onebody_sh_1_1>(*t);
@@ -469,32 +499,29 @@ void try_main (int argc, char* argv[])
 #endif
 #ifdef INCLUDE_ONEBODY
 
-// if using compiler without variadic macros make sure to update the number of elements below
-#define BOOST_PP_ONEBODY_TASK_TUPLE ("overlap",               \
-                                     "kinetic",               \
-                                     "elecpot",               \
-                                     "1emultipole",           \
-                                     "2emultipole",           \
-                                     "3emultipole"            \
+// overlap, kinetic, elecpot cannot be omitted
+#define BOOST_PP_ONEBODY_TASK_TUPLE (overlap,               \
+                                     kinetic,               \
+                                     elecpot,               \
+                                     1emultipole,           \
+                                     2emultipole,           \
+                                     3emultipole,           \
+                                     sphemultipole    \
                                     )
 #define BOOST_PP_ONEBODY_TASK_OPER_TUPLE (OverlapOper,                    \
                                           KineticOper,                    \
                                           ElecPotOper,                    \
                                           CartesianMultipoleOper<3u>,     \
                                           CartesianMultipoleOper<3u>,     \
-                                          CartesianMultipoleOper<3u>      \
+                                          CartesianMultipoleOper<3u>,     \
+                                          SphericalMultipoleOper          \
                                          )
-#if not BOOST_PP_VARIADICS  // no variadic macros? you must MANUALLY specify the number of elements in the tuple here
-#  define BOOST_PP_ONEBODY_TASK_LIST BOOST_PP_TUPLE_TO_LIST( 6, BOOST_PP_ONEBODY_TASK_TUPLE )
-#  define BOOST_PP_ONEBODY_TASK_OPER_LIST BOOST_PP_TUPLE_TO_LIST( 6, BOOST_PP_ONEBODY_TASK_OPER_TUPLE )
-#else
-#  define BOOST_PP_ONEBODY_TASK_LIST BOOST_PP_TUPLE_TO_LIST( BOOST_PP_ONEBODY_TASK_TUPLE )
-#  define BOOST_PP_ONEBODY_TASK_OPER_LIST BOOST_PP_TUPLE_TO_LIST( BOOST_PP_ONEBODY_TASK_OPER_TUPLE )
-#endif
+#define BOOST_PP_ONEBODY_TASK_LIST BOOST_PP_TUPLE_TO_LIST( BOOST_PP_ONEBODY_TASK_TUPLE )
+#define BOOST_PP_ONEBODY_TASK_OPER_LIST BOOST_PP_TUPLE_TO_LIST( BOOST_PP_ONEBODY_TASK_OPER_TUPLE )
 
   for(unsigned int d=0; d<=INCLUDE_ONEBODY; ++d) {
 #define BOOST_PP_ONEBODY_MCR1(r,data,elem)          \
-    taskmgr.add( task_label(elem,d) );
+    taskmgr.add( task_label(BOOST_PP_STRINGIZE(elem),d) );
 
 BOOST_PP_LIST_FOR_EACH ( BOOST_PP_ONEBODY_MCR1, _, BOOST_PP_ONEBODY_TASK_LIST)
 #undef BOOST_PP_ONEBODY_MCR1
@@ -574,7 +601,7 @@ BOOST_PP_LIST_FOR_EACH ( BOOST_PP_ONEBODY_MCR1, _, BOOST_PP_ONEBODY_TASK_LIST)
   }
   for(unsigned int d=0; d<=INCLUDE_ONEBODY; ++d) {
 #define BOOST_PP_ONEBODY_MCR6(r,data,elem)          \
-    cparams->num_bf(task_label(elem, d),     2);
+    cparams->num_bf(task_label(BOOST_PP_STRINGIZE(elem), d),     2);
     BOOST_PP_LIST_FOR_EACH ( BOOST_PP_ONEBODY_MCR6, _, BOOST_PP_ONEBODY_TASK_LIST)
 #   undef BOOST_PP_ONEBODY_MCR6
   }
@@ -743,20 +770,29 @@ BOOST_PP_LIST_FOR_EACH ( BOOST_PP_ONEBODY_MCR1, _, BOOST_PP_ONEBODY_TASK_LIST)
   iface->to_params(iface->macro_define("SHELLQUARTET_SET_ORCA",LIBINT_SHELL_SET_ORCA));
 #if defined(LIBINT_MAX_AM_LIST)
   for(unsigned int d=0; d<ntokens(LIBINT_MAX_AM_LIST,','); ++d) {
-    std::ostringstream oss;
-    oss << "MAX_AM";
-    if (d > 0) oss << d;
-    iface->to_params(iface->macro_define(oss.str(),token<unsigned int>(LIBINT_MAX_AM_LIST,',',d)));
+    {
+      std::ostringstream oss;
+      oss << "MAX_AM";
+      if (d > 0) oss << d;
+      iface->to_params(iface->macro_define(oss.str(),token<unsigned int>(LIBINT_MAX_AM_LIST,',',d)));
+    }
+    {
+      std::ostringstream oss;
+      oss << "MAX_AM_default";
+      if (d > 0) oss << d;
+      iface->to_params(iface->macro_define(oss.str(),token<unsigned int>(LIBINT_MAX_AM_LIST,',',d)));
+    }
   }
 #else
   iface->to_params(iface->macro_define("MAX_AM",LIBINT_MAX_AM));
+  iface->to_params(iface->macro_define("MAX_AM_default",LIBINT_MAX_AM));
 #endif
   cparams->print(os);
 
 #ifdef INCLUDE_ONEBODY
   for(unsigned int d=0; d<=INCLUDE_ONEBODY; ++d) {
 #   define BOOST_PP_ONEBODY_MCR7(r,data,i,elem)          \
-    build_onebody_1b_1k< BOOST_PP_LIST_AT (BOOST_PP_ONEBODY_TASK_OPER_LIST, i) >(os,elem,cparams,iface,d);
+    build_onebody_1b_1k< BOOST_PP_LIST_AT (BOOST_PP_ONEBODY_TASK_OPER_LIST, i) >(os,BOOST_PP_STRINGIZE(elem),cparams,iface,d);
     BOOST_PP_LIST_FOR_EACH_I ( BOOST_PP_ONEBODY_MCR7, _, BOOST_PP_ONEBODY_TASK_LIST)
 #   undef BOOST_PP_ONEBODY_MCR7
   }
@@ -829,8 +865,9 @@ print_header(std::ostream& os)
 {
   os << "----------------------------------------------" << endl;
   os << " build_libint2: optimizing integrals compiler " << endl;
-  os << "                by Edward F. Valeev           " << endl;
-  os << "                and ideas by countless others " << endl;
+  os << "                        by Edward F. Valeev           " << endl;
+  os << "                and ideas by Justin Fermann   "
+     << "                         and Curtis Janssen   " << endl;
   os << "----------------------------------------------" << endl << endl;
 }
 
@@ -882,7 +919,6 @@ build_TwoPRep_2b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
                     SafePtr<Libint2Iface>& iface, unsigned int deriv_level)
 {
   const std::string task = task_label("eri", deriv_level);
-  const std::string task_uc = task_label("ERI", deriv_level);
   typedef TwoPRep_11_11_sq TwoPRep_sh_11_11;
   vector<CGShell*> shells;
   unsigned int lmax = cparams->max_am(task);
@@ -893,7 +929,7 @@ build_TwoPRep_2b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
 
   LibraryTaskManager& taskmgr = LibraryTaskManager::Instance();
   taskmgr.current(task);
-  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task_uc,lmax));
+  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task,lmax));
 
   //
   // Construct graphs for each desired target integral and
@@ -932,7 +968,7 @@ build_TwoPRep_2b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
           const bool need_to_optimize = (max_am <= cparams->max_am_opt(task));
           const bool need_to_unroll = l_to_cgshellsize(la)*l_to_cgshellsize(lb)*
                                       l_to_cgshellsize(lc)*l_to_cgshellsize(ld) <= cparams->unroll_threshold();
-          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? 1000000000 : 1;
+          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? std::numeric_limits<unsigned int>::max() : 0;
           dg_xxxx->registry()->unroll_threshold(unroll_threshold);
           dg_xxxx->registry()->do_cse(need_to_optimize);
           dg_xxxx->registry()->condense_expr(condense_expr(cparams->unroll_threshold(),cparams->max_vector_length()>1));
@@ -947,11 +983,8 @@ build_TwoPRep_2b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
           ////////////
           // loop over unique derivative index combinations
           ////////////
-          // skip 1 center -- all derivatives with respect to that center can be
-          // recovered using translational invariance conditions
-          // which center to skip? -> A = 0, B = 1, C = 2, D = 3
-          const unsigned int center_to_skip = 2;
-          DerivIndexIterator<3> diter(deriv_level);
+          // NB translational invariance is now handled by CR_DerivGauss
+          CartesianDerivIterator<4> diter(deriv_level);
           std::vector< SafePtr<TwoPRep_sh_11_11> > targets;
           bool last_deriv = false;
           do {
@@ -960,17 +993,13 @@ build_TwoPRep_2b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
             CGShell c(lc);
             CGShell d(ld);
 
-            unsigned int center = 0;
             for(unsigned int i=0; i<4; ++i) {
-              if (i == center_to_skip)
-                continue;
               for(unsigned int xyz=0; xyz<3; ++xyz) {
-                if (i == 0) a.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 1) b.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 2) c.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 3) d.deriv().inc(xyz, diter.value(3 * center + xyz));
+                if (i == 0) a.deriv().inc(xyz, (*diter).at(3 * i + xyz));
+                if (i == 1) b.deriv().inc(xyz, (*diter).at(3 * i + xyz));
+                if (i == 2) c.deriv().inc(xyz, (*diter).at(3 * i + xyz));
+                if (i == 3) d.deriv().inc(xyz, (*diter).at(3 * i + xyz));
               }
-              ++center;
             }
 
             SafePtr<TwoPRep_sh_11_11> abcd = TwoPRep_sh_11_11::Instance(a,b,c,d,mType(0u));
@@ -1065,11 +1094,12 @@ build_TwoPRep_1b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
                     SafePtr<Libint2Iface>& iface, unsigned int deriv_level)
 {
   const std::string task = task_label("3eri", deriv_level);
-  const std::string task_uc = task_label("3ERI", deriv_level);
   typedef TwoPRep_11_11_sq TwoPRep_sh_11_11;
   vector<CGShell*> shells;
   const unsigned int lmax = cparams->max_am(task);
   const unsigned int lmax_default = const_cast<const CompilationParameters*>(cparams.get())->max_am(task, 1);
+  if (lmax != lmax_default)
+    iface->to_params(iface->macro_define( std::string("CENTER_DEPENDENT_MAX_AM_") + task, 1));
 
   for(unsigned int l=0; l<=lmax; l++) {
     shells.push_back(new CGShell(l));
@@ -1078,7 +1108,7 @@ build_TwoPRep_1b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
 
   LibraryTaskManager& taskmgr = LibraryTaskManager::Instance();
   taskmgr.current(task);
-  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task_uc,lmax));
+  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task,lmax));
 
   //
   // Construct graphs for each desired target integral and
@@ -1123,7 +1153,7 @@ build_TwoPRep_1b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
           const bool need_to_unroll = l_to_cgshellsize(lbra)*
                                       l_to_cgshellsize(lc)*
                                       l_to_cgshellsize(ld) <= cparams->unroll_threshold();
-          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? 1000000000 : 1;
+          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? std::numeric_limits<unsigned int>::max() : 0;
           dg_xxx->registry()->unroll_threshold(unroll_threshold);
           dg_xxx->registry()->do_cse(need_to_optimize);
           dg_xxx->registry()->condense_expr(condense_expr(cparams->unroll_threshold(),cparams->max_vector_length()>1));
@@ -1134,11 +1164,8 @@ build_TwoPRep_1b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
           ////////////
           // loop over unique derivative index combinations
           ////////////
-          // skip 1 center -- all derivatives with respect to that center can be
-          // recovered using translational invariance conditions
-          // which center to skip? -> the non-dummy one in bra -> A = 0, B = 1
-          const unsigned int center_to_skip = (dummy_center == 0) ? 1 : 0;
-          DerivIndexIterator<2> diter(deriv_level);
+          // NB translational invariance is now handled by CR_DerivGauss
+          CartesianDerivIterator<3> diter(deriv_level);
           std::vector< SafePtr<TwoPRep_sh_11_11> > targets;
           bool last_deriv = false;
           do {
@@ -1153,13 +1180,13 @@ build_TwoPRep_1b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
 
             unsigned int center = 0;
             for(unsigned int i=0; i<4; ++i) {
-              if (i == center_to_skip || i == dummy_center)
+              if (i == dummy_center)
                 continue;
               for(unsigned int xyz=0; xyz<3; ++xyz) {
-                if (i == 0) a.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 1) b.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 2) c.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 3) d.deriv().inc(xyz, diter.value(3 * center + xyz));
+                if (i == 0) a.deriv().inc(xyz, (*diter).at(3 * center + xyz));
+                if (i == 1) b.deriv().inc(xyz, (*diter).at(3 * center + xyz));
+                if (i == 2) c.deriv().inc(xyz, (*diter).at(3 * center + xyz));
+                if (i == 3) d.deriv().inc(xyz, (*diter).at(3 * center + xyz));
               }
               ++center;
             }
@@ -1206,6 +1233,8 @@ build_TwoPRep_1b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
             label += abcd_label;
           }
 
+          std::cout << "working on " << label << " ... "; std::cout.flush();
+
           std::string prefix(cparams->source_directory());
           std::deque<std::string> decl_filenames;
           std::deque<std::string> def_filenames;
@@ -1256,7 +1285,6 @@ build_TwoPRep_1b_1k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
                     SafePtr<Libint2Iface>& iface, unsigned int deriv_level)
 {
   const std::string task = task_label("2eri", deriv_level);
-  const std::string task_uc = task_label("2ERI", deriv_level);
   typedef TwoPRep_11_11_sq TwoPRep_sh_11_11;
   vector<CGShell*> shells;
   unsigned int lmax = cparams->max_am(task);
@@ -1267,7 +1295,7 @@ build_TwoPRep_1b_1k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
 
   LibraryTaskManager& taskmgr = LibraryTaskManager::Instance();
   taskmgr.current(task);
-  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task_uc,lmax));
+  iface->to_params(iface->macro_define( std::string("MAX_AM_") + task,lmax));
 
   //
   // Construct graphs for each desired target integral and
@@ -1309,7 +1337,7 @@ build_TwoPRep_1b_1k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
           const bool need_to_optimize = (max_am <= cparams->max_am_opt(task));
           const bool need_to_unroll = l_to_cgshellsize(lbra)*
                                       l_to_cgshellsize(lket) <= cparams->unroll_threshold();
-          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? 1000000000 : 1;
+          const unsigned int unroll_threshold = need_to_optimize && need_to_unroll ? std::numeric_limits<unsigned int>::max() : 0;
           dg_xxx->registry()->unroll_threshold(unroll_threshold);
           dg_xxx->registry()->do_cse(need_to_optimize);
           dg_xxx->registry()->condense_expr(condense_expr(cparams->unroll_threshold(),cparams->max_vector_length()>1));
@@ -1319,11 +1347,8 @@ build_TwoPRep_1b_1k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
           ////////////
           // loop over unique derivative index combinations
           ////////////
-          // skip 1 center -- all derivatives with respect to that center can be
-          // recovered using translational invariance conditions
-          // which center to skip? -> the non-dummy one in bra -> A = 0, B = 1
-          const unsigned int center_to_skip = (dummy_center1 == 0) ? 1 : 0;
-          DerivIndexIterator<1> diter(deriv_level);
+          // NB translational invariance is now handled by CR_DerivGauss
+          CartesianDerivIterator<2> diter(deriv_level);
           std::vector< SafePtr<TwoPRep_sh_11_11> > targets;
           bool last_deriv = false;
           do {
@@ -1340,13 +1365,13 @@ build_TwoPRep_1b_1k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
 
             unsigned int center = 0;
             for(unsigned int i=0; i<4; ++i) {
-              if (i == center_to_skip || i == dummy_center1 || i == dummy_center2)
+              if (i == dummy_center1 || i == dummy_center2)
                 continue;
               for(unsigned int xyz=0; xyz<3; ++xyz) {
-                if (i == 0) a.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 1) b.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 2) c.deriv().inc(xyz, diter.value(3 * center + xyz));
-                if (i == 3) d.deriv().inc(xyz, diter.value(3 * center + xyz));
+                if (i == 0) a.deriv().inc(xyz, (*diter).at(3 * center + xyz));
+                if (i == 1) b.deriv().inc(xyz, (*diter).at(3 * center + xyz));
+                if (i == 2) c.deriv().inc(xyz, (*diter).at(3 * center + xyz));
+                if (i == 3) d.deriv().inc(xyz, (*diter).at(3 * center + xyz));
               }
               ++center;
             }
@@ -1395,6 +1420,8 @@ build_TwoPRep_1b_1k(std::ostream& os, const SafePtr<CompilationParameters>& cpar
             label += abcd_label;
           }
 
+          std::cout << "working on " << label << " ... "; std::cout.flush();
+
           std::string prefix(cparams->source_directory());
           std::deque<std::string> decl_filenames;
           std::deque<std::string> def_filenames;
@@ -1941,29 +1968,92 @@ build_G12DKH_2b_2k(std::ostream& os, const SafePtr<CompilationParameters>& cpara
 void
 config_to_api(const SafePtr<CompilationParameters>& cparams, SafePtr<Libint2Iface>& iface)
 {
+  int max_deriv_order = 0;
 #ifdef INCLUDE_ONEBODY
   iface->to_params(iface->macro_define("SUPPORT_ONEBODY",1));
   iface->to_params(iface->macro_define("DERIV_ONEBODY_ORDER",INCLUDE_ONEBODY));
+#ifdef DISABLE_ONEBODY_PROPERTY_DERIVS
+  iface->to_params(iface->macro_define("DERIV_ONEBODY_PROPERTY_ORDER",0));
+#else
+  iface->to_params(iface->macro_define("DERIV_ONEBODY_PROPERTY_ORDER",INCLUDE_ONEBODY));
+#endif
+  max_deriv_order = std::max(max_deriv_order,INCLUDE_ONEBODY);
 #endif
 #ifdef INCLUDE_ERI
   iface->to_params(iface->macro_define("SUPPORT_ERI",1));
   iface->to_params(iface->macro_define("DERIV_ERI_ORDER",INCLUDE_ERI));
+  max_deriv_order = std::max(max_deriv_order,INCLUDE_ERI);
 #endif
 #ifdef INCLUDE_ERI3
   iface->to_params(iface->macro_define("SUPPORT_ERI3",1));
   iface->to_params(iface->macro_define("DERIV_ERI3_ORDER",INCLUDE_ERI3));
+  max_deriv_order = std::max(max_deriv_order,INCLUDE_ERI3);
 #endif
 #ifdef INCLUDE_ERI2
   iface->to_params(iface->macro_define("SUPPORT_ERI2",1));
   iface->to_params(iface->macro_define("DERIV_ERI2_ORDER",INCLUDE_ERI2));
+  max_deriv_order = std::max(max_deriv_order,INCLUDE_ERI2);
 #endif
 #ifdef INCLUDE_G12
   iface->to_params(iface->macro_define("SUPPORT_G12",1));
   iface->to_params(iface->macro_define("DERIV_G12_ORDER",INCLUDE_G12));
+  max_deriv_order = std::max(max_deriv_order,INCLUDE_G12);
 #endif
 #ifdef INCLUDE_G12DKH
   iface->to_params(iface->macro_define("SUPPORT_G12DKH",1));
   iface->to_params(iface->macro_define("DERIV_G12DKH_ORDER",INCLUDE_G12DKH));
+  max_deriv_order = std::max(max_deriv_order,INCLUDE_G12DKH);
 #endif
+  iface->to_params(iface->macro_define("MAX_DERIV_ORDER",max_deriv_order));
+
+  // this is only needed for preprocessor-based generic processing of all generated tasks
+  // declare all tasks in a range of valid tasks as defined or not
+  LibraryTaskManager& taskmgr = LibraryTaskManager::Instance();
+  // the range is defined by max # of centers, max deriv order, and operator set
+  const size_t max_ncenter = 4;
+  for(unsigned int ncenter=0; ncenter<=max_ncenter; ++ncenter) {
+
+    std::stringstream oss;
+    oss << ncenter;
+
+    for(unsigned int d=0; d<=max_deriv_order; ++d) {
+      std::string abbrv_label, full_label;
+
+      { // 1-body ints
+        std::string ncenter_str = oss.str();
+        std::string ncenter_str_abbrv = ncenter == 2 ? std::string("") : oss.str();
+#define BOOST_PP_MCR1(r,data,elem)                                   \
+        abbrv_label = task_label(ncenter_str_abbrv + BOOST_PP_STRINGIZE(elem),d);        \
+        full_label = task_label(ncenter_str + BOOST_PP_STRINGIZE(elem),d);               \
+        iface->to_params(iface->macro_define(std::string("TASK_EXISTS_") + full_label,taskmgr.exists(abbrv_label) ? 1 : 0));
+
+BOOST_PP_LIST_FOR_EACH ( BOOST_PP_MCR1, _, BOOST_PP_ONEBODY_TASK_LIST)
+#undef BOOST_PP_MCR1
+      }
+
+      { // 2-body ints
+
+#define BOOST_PP_TWOBODY_TASKOPER_TUPLE ("eri",               \
+                                         "r12kg12",           \
+                                         "r12_0_g12",         \
+                                         "r12_2_g12",         \
+                                         "g12_T1_g12",        \
+                                         "g12dkh"             \
+        )
+#define BOOST_PP_TWOBODY_TASKOPER_LIST BOOST_PP_TUPLE_TO_LIST( BOOST_PP_TWOBODY_TASKOPER_TUPLE )
+
+        std::string ncenter_str = oss.str();
+        std::string ncenter_str_abbrv = ncenter == 4 ? std::string("") : oss.str();
+#define BOOST_PP_MCR1(r,data,elem)                                   \
+        abbrv_label = task_label(ncenter_str_abbrv + elem,d);        \
+        full_label = task_label(ncenter_str + elem,d);               \
+        iface->to_params(iface->macro_define(std::string("TASK_EXISTS_") + full_label,taskmgr.exists(abbrv_label) ? 1 : 0));
+
+BOOST_PP_LIST_FOR_EACH ( BOOST_PP_MCR1, _, BOOST_PP_TWOBODY_TASKOPER_LIST)
+#undef BOOST_PP_MCR1
+      }
+    }
+  }
+
 }
 
