@@ -1,19 +1,20 @@
 /*
- *  This file is a part of Libint.
- *  Copyright (C) 2004-2014 Edward F. Valeev
+ *  Copyright (C) 2004-2019 Edward F. Valeev
  *
- *  This program is free software: you can redistribute it and/or modify
+ *  This file is part of Libint.
+ *
+ *  Libint is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Libint is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see http://www.gnu.org/licenses/.
+ *  along with Libint.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,6 +32,9 @@
 #include <master_ints_list.h>
 #include <master_rrs_list.h>
 
+#include <integral_1_1.impl.h>
+#include <integral_11_11.impl.h>
+
 // MPL is painful
 #include <boost/mpl/list.hpp>
 #include <boost/mpl/for_each.hpp>
@@ -62,7 +66,7 @@ namespace libint2 {
   template <class T> struct MasterStrategy;
 #if LIBINT_SHELLQUARTET_STRATEGY == LIBINT_SHELLQUARTET_STRATEGY_A0C0
   template <> struct MasterStrategy<TwoPRep_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_TwoPRep_11_sh,
     HRR_cd_11_TwoPRep_11_sh,
     Deriv_a_11_TwoPRep_11_sh,
@@ -78,7 +82,7 @@ namespace libint2 {
     > value;
   };
   template <> struct MasterStrategy<TwoPRep_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_TwoPRep_11_int,
     HRR_cd_11_TwoPRep_11_int,
     Deriv_a_11_TwoPRep_11_int,
@@ -95,7 +99,7 @@ namespace libint2 {
   };
 #else  // 0B0D strategy
   template <> struct MasterStrategy<TwoPRep_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ba_11_TwoPRep_11_sh,
     HRR_dc_11_TwoPRep_11_sh,
     Deriv_a_11_TwoPRep_11_sh,
@@ -111,7 +115,7 @@ namespace libint2 {
     > value;
   };
   template <> struct MasterStrategy<TwoPRep_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ba_11_TwoPRep_11_int,
     HRR_dc_11_TwoPRep_11_int,
     Deriv_a_11_TwoPRep_11_int,
@@ -130,7 +134,7 @@ namespace libint2 {
 
 #if LIBINT_SHELLQUARTET_STRATEGY == LIBINT_SHELLQUARTET_STRATEGY_A0C0
   template <> struct MasterStrategy<R12kG12_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_R12kG12_11_sh,
     HRR_cd_11_R12kG12_11_sh,
     VRR_a_11_R12kG12_11_sh,
@@ -138,7 +142,7 @@ namespace libint2 {
     > value;
   };
   template <> struct MasterStrategy<R12kG12_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_R12kG12_11_int,
     HRR_cd_11_R12kG12_11_int,
     VRR_a_11_R12kG12_11_int,
@@ -147,7 +151,7 @@ namespace libint2 {
   };
 #else // 0B0D strategy
   template <> struct MasterStrategy<R12kG12_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ba_11_R12kG12_11_sh,
     HRR_dc_11_R12kG12_11_sh,
     VRR_b_11_R12kG12_11_sh,
@@ -155,7 +159,7 @@ namespace libint2 {
     > value;
   };
   template <> struct MasterStrategy<R12kG12_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ba_11_R12kG12_11_int,
     HRR_dc_11_R12kG12_11_int,
     VRR_b_11_R12kG12_11_int,
@@ -165,56 +169,56 @@ namespace libint2 {
 #endif
 
   template <> struct MasterStrategy<R12kR12lG12_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_R12kR12lG12_11_sh,
     HRR_cd_11_R12kR12lG12_11_sh,
     CR_11_R12kR12lG12_11_sh
     > value;
   };
   template <> struct MasterStrategy<R12kR12lG12_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_R12kR12lG12_11_int,
     HRR_cd_11_R12kR12lG12_11_int,
     CR_11_R12kR12lG12_11_int
     > value;
   };
   template <> struct MasterStrategy<TiG12_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_TiG12_11_sh,
     HRR_cd_11_TiG12_11_sh,
     CR_11_TiG12_11_sh
     > value;
   };
   template <> struct MasterStrategy<TiG12_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_TiG12_11_int,
     HRR_cd_11_TiG12_11_int,
     CR_11_TiG12_11_int
     > value;
   };
   template <> struct MasterStrategy<G12TiG12_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_G12TiG12_11_sh,
     HRR_cd_11_G12TiG12_11_sh,
     CR_11_G12TiG12_11_sh
     > value;
   };
   template <> struct MasterStrategy<G12TiG12_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_G12TiG12_11_int,
     HRR_cd_11_G12TiG12_11_int,
     CR_11_G12TiG12_11_int
     > value;
   };
   template <> struct MasterStrategy<DivG12prime_xTx_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_DivG12prime_xTx_sh,
     HRR_cd_11_DivG12prime_xTx_sh,
     CR_11_DivG12prime_xTx_11_sh
     > value;
   };
   template <> struct MasterStrategy<DivG12prime_xTx_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_DivG12prime_xTx_int,
     HRR_cd_11_DivG12prime_xTx_int,
     CR_11_DivG12prime_xTx_11_int
@@ -222,26 +226,26 @@ namespace libint2 {
   };
 #if LIBINT_SHELLQUARTET_STRATEGY == LIBINT_SHELLQUARTET_STRATEGY_A0C0
   template <> struct MasterStrategy<DummySymmIntegral_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_Dummy_11_sh,
     HRR_cd_11_Dummy_11_sh
     > value;
   };
   template <> struct MasterStrategy<DummySymmIntegral_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ab_11_Dummy_11_int,
     HRR_cd_11_Dummy_11_int
     > value;
   };
 #else // 0B0D strategy
   template <> struct MasterStrategy<DummySymmIntegral_11_11_sq> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ba_11_Dummy_11_sh,
     HRR_dc_11_Dummy_11_sh
     > value;
   };
   template <> struct MasterStrategy<DummySymmIntegral_11_11_int> {
-    typedef mpl::list<
+    typedef boost::mpl::list<
     HRR_ba_11_Dummy_11_int,
     HRR_dc_11_Dummy_11_int
     > value;
@@ -250,131 +254,137 @@ namespace libint2 {
 
 #if LIBINT_SUPPORT_ONEBODYINTS
   template <> struct MasterStrategy<Overlap_1_1_sh_x> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGShell1d<CartesianAxis_X>,OverlapOper>
       > value;
     };
   template <> struct MasterStrategy<Overlap_1_1_sh_y> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGShell1d<CartesianAxis_Y>,OverlapOper>
       > value;
     };
   template <> struct MasterStrategy<Overlap_1_1_sh_z> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGShell1d<CartesianAxis_Z>,OverlapOper>
       > value;
     };
   template <> struct MasterStrategy<Overlap_1_1_int_x> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
       VRR_a_1_Overlap_1_int_x,
       VRR_b_1_Overlap_1_int_x
       > value;
     };
   template <> struct MasterStrategy<Overlap_1_1_int_y> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
       VRR_a_1_Overlap_1_int_y,
       VRR_b_1_Overlap_1_int_y
       > value;
     };
   template <> struct MasterStrategy<Overlap_1_1_int_z> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
       VRR_a_1_Overlap_1_int_z,
       VRR_b_1_Overlap_1_int_z
       > value;
     };
-# if LIBINT_SHELLQUARTET_STRATEGY == LIBINT_SHELLQUARTET_STRATEGY_A0C0
+  // TODO create TwoCenterOSTactic to be able to optimally apply bidirectional strategies
   template <> struct MasterStrategy<ElecPot_1_1_sh> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
       HRR_ab_1_ElecPot_1_sh,
-      CR_DerivGauss<ElecPot_1_1_sh,0,InBra>,
-      CR_DerivGauss<ElecPot_1_1_sh,0,InKet>,
-      VRR_a_1_ElecPot_1_sh
-      > value;
-    };
-  template <> struct MasterStrategy<ElecPot_1_1_int> {
-      typedef mpl::list<
-      HRR_ab_1_ElecPot_1_int,
-      CR_DerivGauss<ElecPot_1_1_int,0,InBra>,
-      CR_DerivGauss<ElecPot_1_1_int,0,InKet>,
-      VRR_a_1_ElecPot_1_int
-      > value;
-    };
-# else //  // 0B0D strategy
-  template <> struct MasterStrategy<ElecPot_1_1_sh> {
-      typedef mpl::list<
       HRR_ba_1_ElecPot_1_sh,
       CR_DerivGauss<ElecPot_1_1_sh,0,InBra>,
       CR_DerivGauss<ElecPot_1_1_sh,0,InKet>,
+      VRR_a_1_ElecPot_1_sh,
       VRR_b_1_ElecPot_1_sh
       > value;
     };
   template <> struct MasterStrategy<ElecPot_1_1_int> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
+      HRR_ab_1_ElecPot_1_int,
       HRR_ba_1_ElecPot_1_int,
       CR_DerivGauss<ElecPot_1_1_int,0,InBra>,
       CR_DerivGauss<ElecPot_1_1_int,0,InKet>,
+      VRR_a_1_ElecPot_1_int,
       VRR_b_1_ElecPot_1_int
       > value;
     };
-# endif // strategy: A0 or 0B
   template <> struct MasterStrategy<Kinetic_1_1_sh> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGShell,KineticOper>
       > value;
     };
   template <> struct MasterStrategy<Kinetic_1_1_int> {
-      typedef mpl::list<
-        CR_DerivGauss<Kinetic_1_1_int,0,InBra>,
-        CR_DerivGauss<Kinetic_1_1_int,0,InKet>,
+      typedef boost::mpl::list<
+        CR_DerivGauss<Kinetic_1_1_int,0,InBra,trinvskip1_part,trinvskip1_where>,
+        CR_DerivGauss<Kinetic_1_1_int,0,InKet,trinvskip1_part,trinvskip1_where>,
         CR_XYZ_1_1<CGF,KineticOper>
       > value;
     };
   template <> struct MasterStrategy<Kinetic_1_1_int_x> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGF1d<CartesianAxis_X>,KineticOper>
       > value;
     };
   template <> struct MasterStrategy<Kinetic_1_1_int_y> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGF1d<CartesianAxis_Y>,KineticOper>
       > value;
     };
   template <> struct MasterStrategy<Kinetic_1_1_int_z> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGF1d<CartesianAxis_Z>,KineticOper>
       > value;
     };
   template <> struct MasterStrategy<CMultipole_1_1_sh> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGShell,CartesianMultipoleOper<3u>>
       > value;
     };
   template <> struct MasterStrategy<CMultipole_1_1_int> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_DerivGauss<CMultipole_1_1_int,0,InBra>,
         CR_DerivGauss<CMultipole_1_1_int,0,InKet>,
         CR_XYZ_1_1<CGF,CartesianMultipoleOper<3u>>
       > value;
     };
   template <> struct MasterStrategy<CMultipole_1_1_int_x> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGF1d<CartesianAxis_X>,CartesianMultipoleOper<1u>>
       > value;
     };
   template <> struct MasterStrategy<CMultipole_1_1_int_y> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGF1d<CartesianAxis_Y>,CartesianMultipoleOper<1u>>
       > value;
     };
   template <> struct MasterStrategy<CMultipole_1_1_int_z> {
-      typedef mpl::list<
+      typedef boost::mpl::list<
         CR_XYZ_1_1<CGF1d<CartesianAxis_Z>,CartesianMultipoleOper<1u>>
       > value;
     };
+  template <> struct MasterStrategy<SMultipole_1_1_sh> {
+      typedef boost::mpl::list<
+        HRR_ab_1_SMultipole_1_sh,
+        HRR_ba_1_SMultipole_1_sh,
+        CR_DerivGauss<SMultipole_1_1_sh,0,InBra>,
+        CR_DerivGauss<SMultipole_1_1_sh,0,InKet>,
+        VRR_a_1_SMultipole_1_sh,
+        VRR_b_1_SMultipole_1_sh
+      > value;
+    };
+  template <> struct MasterStrategy<SMultipole_1_1_int> {
+      typedef boost::mpl::list<
+          HRR_ab_1_SMultipole_1_int,
+          HRR_ba_1_SMultipole_1_int,
+          CR_DerivGauss<SMultipole_1_1_int,0,InBra>,
+          CR_DerivGauss<SMultipole_1_1_int,0,InKet>,
+          VRR_a_1_SMultipole_1_int,
+          VRR_b_1_SMultipole_1_int
+      > value;
+    };
 #endif // LIBINT_SUPPORT_ONEBODYINTS
 
   template <typename T> struct MasterStrategy {
-      typedef mpl::list<> value;
+      typedef boost::mpl::list<> value;
   };
 
   /// transform<RRType> encapsulates RRType and the action associated with RRType
@@ -509,10 +519,12 @@ namespace libint2 {
 #endif
         using namespace boost;
         using namespace boost::mpl::placeholders;
-        // Try to unroll first, if this is a shell set
+        // If this is a shell set, try unrolling if
+        // 1. size > unroll_size, or
+        // 2. auto_unroll is true
         const unsigned int size = integral->size();
         const bool can_unroll = not TrivialBFSet<typename T::BasisFunctionType>::result &&
-                                (size <= dg->registry()->unroll_threshold());
+            (size <= dg->registry()->unroll_threshold() || tptr->auto_unroll());
 //        std::cout << "  size=" << size << " can unroll? " << (can_unroll ? "yes" : "no") << std::endl;
 //        if (not can_unroll) {
 //          std::cout << "    unroll_threshold=" << dg->registry()->unroll_threshold()
