@@ -1,19 +1,20 @@
 /*
- *  This file is a part of Libint.
- *  Copyright (C) 2004-2014 Edward F. Valeev
+ *  Copyright (C) 2004-2019 Edward F. Valeev
  *
- *  This program is free software: you can redistribute it and/or modify
+ *  This file is part of Libint.
+ *
+ *  Libint is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Libint is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see http://www.gnu.org/licenses/.
+ *  along with Libint.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 #include "../../../../../SIMD_wrapped_vector/template/simd_wrapped_vector.hpp"
@@ -22,7 +23,7 @@
 
 #include <rr.h>
 #include <iter.h>
-#include <deriv_iter.h>
+#include <libint2/deriv_iter.h>
 #include <policy_spec.h>
 #include <global_macros.h>
 #include <cgshell_ordering.h>
@@ -31,18 +32,18 @@
 #define _libint2_libint2types_h_
 
 #define LIBINT2_MAX_VECLEN 1
-constexpr unsigned int am0 = 2;
-constexpr unsigned int am1 = 2;
-constexpr unsigned int am2 = 2;
-constexpr unsigned int am3 = 2;
-constexpr unsigned int am_tot = am0 + am1 + am2 + am3;
+constexpr auto am0 = 1u;
+constexpr auto am1 = 0u;
+constexpr auto am2 = 1u;
+constexpr auto am3 = 0u;
+constexpr auto am_tot = am0 + am1 + am2 + am3;
 int bool_test = 0;
 
 
-#include <libint2/vector.h>
-#include <libint2/memory.h>
-#include <libint2/intrinsic_operations.h>
-#include <libint2/timer.h>
+#include <libint2/util/vector.h>
+#include <libint2/util/memory.h>
+#include <libint2/util/intrinsic_operations.h>
+#include <libint2/util/timer.h>
 typedef struct {
     LIBINT2_REALTYPE AB_x[LIBINT2_MAX_VECLEN];
     LIBINT2_REALTYPE AB_y[LIBINT2_MAX_VECLEN];
@@ -75,7 +76,7 @@ typedef struct {
 
 #include <VRR_GTG_1d_xx_xx.h>
 #include <libint/util.h>
-#include <libint/deriv_iter.h>
+#include <libint2/deriv_iter.h>
 #include <test_eri/eri.h>
 
 #ifdef LIBINT_HAVE_LIBROOTS
@@ -177,7 +178,7 @@ namespace libint2 {
 
 typedef unsigned int uint;
 
-libint2::FmEval_Chebyshev3<double> fmeval_chebyshev(28);
+libint2::FmEval_Chebyshev7<double> fmeval_chebyshev(28);
 libint2::FmEval_Taylor<double,7> fmeval_taylor(28, 1e-15);
 
 int main(int argc, char** argv)
@@ -197,8 +198,8 @@ int main(int argc, char** argv)
   const uint contrdepth4 = contrdepth * contrdepth * contrdepth * contrdepth;
 
   const unsigned int deriv_order = 0;
-  DerivIndexIterator<4> diter(deriv_order);
-  const unsigned int nderiv = diter.range_rank();
+  CartesianDerivIterator<4> diter(deriv_order);
+  const unsigned int nderiv = diter.range_size();
 
   CGShell sh0(am[0]);
   CGShell sh1(am[1]);
@@ -468,11 +469,11 @@ int main(int argc, char** argv)
 
                     const LIBINT2_REF_REALTYPE c0123 = c0[p0] * c1[p1] * c2[p2] * c3[p3];
 
-                    DerivIndexIterator<4> diter(deriv_order);
+                    CartesianDerivIterator<4> diter(deriv_order);
                     bool last_deriv = false;
                     unsigned int di = 0;
                     do {
-                      ref_eri[di++] += c0123 * eri(diter.values(),
+                      ref_eri[di++] += c0123 * eri(&(*diter)[0],
                                                    l0,m0,n0,a0,Aref,
                                                    l1,m1,n1,a1,Bref,
                                                    l2,m2,n2,a2,Cref,
