@@ -1,19 +1,20 @@
 /*
- *  This file is a part of Libint.
- *  Copyright (C) 2004-2014 Edward F. Valeev
+ *  Copyright (C) 2004-2019 Edward F. Valeev
  *
- *  This program is free software: you can redistribute it and/or modify
+ *  This file is part of Libint.
+ *
+ *  Libint is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Libint is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see http://www.gnu.org/licenses/.
+ *  along with Libint.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -32,8 +33,12 @@
 
 // Libint Gaussian integrals library
 #include <libint2.hpp>
+#if !LIBINT2_CONSTEXPR_STATICS
+#  include <libint2/statics_definition.h>
+#endif
 
-typedef Eigen::Matrix<double, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor>
+using real_t = libint2::scalar_type;
+typedef Eigen::Matrix<real_t, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor>
         Matrix;  // import dense, dynamically sized Matrix type from Eigen;
                  // this is a matrix with row-major storage (http://en.wikipedia.org/wiki/Row-major_order)
                  // to meet the layout of the integrals returned by the Libint integral library
@@ -43,22 +48,20 @@ struct Atom {
     double x, y, z;
 };
 
-using libint2::Shell;
-
 std::vector<Atom> read_geometry(const std::string& filename);
-std::vector<Shell> make_sto3g_basis(const std::vector<Atom>& atoms);
-size_t nbasis(const std::vector<Shell>& shells);
-std::vector<size_t> map_shell_to_basis_function(const std::vector<Shell>& shells);
+std::vector<libint2::Shell> make_sto3g_basis(const std::vector<Atom>& atoms);
+size_t nbasis(const std::vector<libint2::Shell>& shells);
+std::vector<size_t> map_shell_to_basis_function(const std::vector<libint2::Shell>& shells);
 Matrix compute_soad(const std::vector<Atom>& atoms);
 Matrix compute_1body_ints(const std::vector<libint2::Shell>& shells,
-                          libint2::OneBodyEngine::operator_type t,
+                          libint2::Operator t,
                           const std::vector<Atom>& atoms = std::vector<Atom>());
 
 // simple-to-read, but inefficient Fock builder; computes ~16 times as many ints as possible
-Matrix compute_2body_fock_simple(const std::vector<Shell>& shells,
+Matrix compute_2body_fock_simple(const std::vector<libint2::Shell>& shells,
                                  const Matrix& D);
 // an efficient Fock builder; *integral-driven* hence computes permutationally-unique ints once
-Matrix compute_2body_fock(const std::vector<Shell>& shells,
+Matrix compute_2body_fock(const std::vector<libint2::Shell>& shells,
                                  const Matrix& D);
 
 int main(int argc, char *argv[]) {
@@ -67,6 +70,10 @@ int main(int argc, char *argv[]) {
   using std::cerr;
   using std::endl;
 
+  using libint2::Shell;
+  using libint2::Engine;
+  using libint2::Operator;
+
   try {
 
     /*** =========================== ***/
@@ -110,20 +117,20 @@ int main(int argc, char *argv[]) {
     /*** =========================== ***/
 
     // initializes the Libint integrals library ... now ready to compute
-    libint2::init();
+    libint2::initialize();
 
     // compute overlap integrals
-    auto S = compute_1body_ints(shells, libint2::OneBodyEngine::overlap);
+    auto S = compute_1body_ints(shells, Operator::overlap);
     cout << "\n\tOverlap Integrals:\n";
     cout << S << endl;
 
     // compute kinetic-energy integrals
-    auto T = compute_1body_ints(shells, libint2::OneBodyEngine::kinetic);
+    auto T = compute_1body_ints(shells, Operator::kinetic);
     cout << "\n\tKinetic-Energy Integrals:\n";
     cout << T << endl;
 
     // compute nuclear-attraction integrals
-    Matrix V = compute_1body_ints(shells, libint2::OneBodyEngine::nuclear, atoms);
+    Matrix V = compute_1body_ints(shells, Operator::nuclear, atoms);
     cout << "\n\tNuclear Attraction Integrals:\n";
     cout << V << endl;
 
@@ -169,11 +176,11 @@ int main(int argc, char *argv[]) {
     /*** =========================== ***/
 
     const auto maxiter = 100;
-    const auto conv = 1e-12;
+    const real_t conv = 1e-12;
     auto iter = 0;
-    auto rmsd = 0.0;
-    auto ediff = 0.0;
-    auto ehf = 0.0;
+    real_t rmsd = 0.0;
+    real_t ediff = 0.0;
+    real_t ehf = 0.0;
     do {
       const auto tstart = std::chrono::high_resolution_clock::now();
       ++iter;
@@ -224,7 +231,7 @@ int main(int argc, char *argv[]) {
 
     printf("** Hartree-Fock energy = %20.12f\n", ehf + enuc);
 
-    libint2::cleanup(); // done with libint
+    libint2::finalize(); // done with libint
 
   } // end of try block; if any exceptions occurred, report them and exit cleanly
 
@@ -285,7 +292,7 @@ std::vector<Atom> read_dotxyz(std::istream& is) {
       Z = 17;
     else {
       std::cerr << "read_dotxyz: element label \"" << element_label << "\" is not recognized" << std::endl;
-      throw "Did not recognize element label in .xyz file";
+      throw std::invalid_argument("Did not recognize element label in .xyz file");
     }
 
     atoms[i].atomic_number = Z;
@@ -319,10 +326,12 @@ std::vector<Atom> read_geometry(const std::string& filename) {
   if ( filename.rfind(".xyz") != std::string::npos)
     return read_dotxyz(iss);
   else
-    throw "only .xyz files are accepted";
+    throw std::invalid_argument("only .xyz files are accepted");
 }
 
-std::vector<Shell> make_sto3g_basis(const std::vector<Atom>& atoms) {
+std::vector<libint2::Shell> make_sto3g_basis(const std::vector<Atom>& atoms) {
+
+  using libint2::Shell;
 
   std::vector<Shell> shells;
 
@@ -444,21 +453,21 @@ std::vector<Shell> make_sto3g_basis(const std::vector<Atom>& atoms) {
   return shells;
 }
 
-size_t nbasis(const std::vector<Shell>& shells) {
+size_t nbasis(const std::vector<libint2::Shell>& shells) {
   size_t n = 0;
   for (const auto& shell: shells)
     n += shell.size();
   return n;
 }
 
-size_t max_nprim(const std::vector<Shell>& shells) {
+size_t max_nprim(const std::vector<libint2::Shell>& shells) {
   size_t n = 0;
   for (auto shell: shells)
     n = std::max(shell.nprim(), n);
   return n;
 }
 
-int max_l(const std::vector<Shell>& shells) {
+int max_l(const std::vector<libint2::Shell>& shells) {
   int l = 0;
   for (auto shell: shells)
     for (auto c: shell.contr)
@@ -466,7 +475,7 @@ int max_l(const std::vector<Shell>& shells) {
   return l;
 }
 
-std::vector<size_t> map_shell_to_basis_function(const std::vector<Shell>& shells) {
+std::vector<size_t> map_shell_to_basis_function(const std::vector<libint2::Shell>& shells) {
   std::vector<size_t> result;
   result.reserve(shells.size());
 
@@ -519,26 +528,33 @@ Matrix compute_soad(const std::vector<Atom>& atoms) {
 }
 
 Matrix compute_1body_ints(const std::vector<libint2::Shell>& shells,
-                          libint2::OneBodyEngine::operator_type obtype,
+                          libint2::Operator obtype,
                           const std::vector<Atom>& atoms)
 {
+  using libint2::Shell;
+  using libint2::Engine;
+  using libint2::Operator;
+
   const auto n = nbasis(shells);
   Matrix result(n,n);
 
   // construct the overlap integrals engine
-  libint2::OneBodyEngine engine(obtype, max_nprim(shells), max_l(shells), 0);
+  Engine engine(obtype, max_nprim(shells), max_l(shells), 0);
   // nuclear attraction ints engine needs to know where the charges sit ...
   // the nuclei are charges in this case; in QM/MM there will also be classical charges
-  if (obtype == libint2::OneBodyEngine::nuclear) {
-    std::vector<std::pair<double,std::array<double,3>>> q;
+  if (obtype == Operator::nuclear) {
+    std::vector<std::pair<real_t,std::array<real_t,3>>> q;
     for(const auto& atom : atoms) {
-      q.push_back( {static_cast<double>(atom.atomic_number), {{atom.x, atom.y, atom.z}}} );
+      q.push_back( {static_cast<real_t>(atom.atomic_number), {{atom.x, atom.y, atom.z}}} );
     }
     engine.set_params(q);
   }
 
   auto shell2bf = map_shell_to_basis_function(shells);
 
+  // buf[0] points to the target shell set after every call  to engine.compute()
+  const auto& buf = engine.results();
+
   // loop over unique shell pairs, {s1,s2} such that s1 >= s2
   // this is due to the permutational symmetry of the real integrals over Hermitian operators: (1|2) = (2|1)
   for(auto s1=0; s1!=shells.size(); ++s1) {
@@ -551,11 +567,11 @@ Matrix compute_1body_ints(const std::vector<libint2::Shell>& shells,
       auto bf2 = shell2bf[s2];
       auto n2 = shells[s2].size();
 
-      // compute shell pair; return is the pointer to the buffer
-      const auto* buf = engine.compute(shells[s1], shells[s2]);
+      // compute shell pair
+      engine.compute(shells[s1], shells[s2]);
 
       // "map" buffer to a const Eigen Matrix, and copy it to the corresponding blocks of the result
-      Eigen::Map<const Matrix> buf_mat(buf, n1, n2);
+      Eigen::Map<const Matrix> buf_mat(buf[0], n1, n2);
       result.block(bf1, bf2, n1, n2) = buf_mat;
       if (s1 != s2) // if s1 >= s2, copy {s1,s2} to the corresponding {s2,s1} block, note the transpose!
       result.block(bf2, bf1, n2, n1) = buf_mat.transpose();
@@ -566,17 +582,24 @@ Matrix compute_1body_ints(const std::vector<libint2::Shell>& shells,
   return result;
 }
 
-Matrix compute_2body_fock_simple(const std::vector<Shell>& shells,
+Matrix compute_2body_fock_simple(const std::vector<libint2::Shell>& shells,
                                  const Matrix& D) {
 
+  using libint2::Shell;
+  using libint2::Engine;
+  using libint2::Operator;
+
   const auto n = nbasis(shells);
   Matrix G = Matrix::Zero(n,n);
 
   // construct the electron repulsion integrals engine
-  libint2::TwoBodyEngine<libint2::Coulomb> engine(max_nprim(shells), max_l(shells), 0);
+  Engine engine(Operator::coulomb, max_nprim(shells), max_l(shells), 0);
 
   auto shell2bf = map_shell_to_basis_function(shells);
 
+  // buf[0] points to the target shell set after every call  to engine.compute()
+  const auto& buf = engine.results();
+
   // loop over shell pairs of the Fock matrix, {s1,s2}
   // Fock matrix is symmetric, but skipping it here for simplicity (see compute_2body_fock)
   for(auto s1=0; s1!=shells.size(); ++s1) {
@@ -602,7 +625,10 @@ Matrix compute_2body_fock_simple(const std::vector<Shell>& shells,
           auto n4 = shells[s4].size();
 
           // Coulomb contribution to the Fock matrix is from {s1,s2,s3,s4} integrals
-          const auto* buf_1234 = engine.compute(shells[s1], shells[s2], shells[s3], shells[s4]);
+          engine.compute(shells[s1], shells[s2], shells[s3], shells[s4]);
+          const auto* buf_1234 = buf[0];
+          if (buf_1234 == nullptr)
+            continue; // if all integrals screened out, skip to next quartet
 
           // we don't have an analog of Eigen for tensors (yet ... see github.com/BTAS/BTAS, under development)
           // hence some manual labor here:
@@ -623,7 +649,8 @@ Matrix compute_2body_fock_simple(const std::vector<Shell>& shells,
           }
 
           // exchange contribution to the Fock matrix is from {s1,s3,s2,s4} integrals
-          const auto* buf_1324 = engine.compute(shells[s1], shells[s3], shells[s2], shells[s4]);
+          engine.compute(shells[s1], shells[s3], shells[s2], shells[s4]);
+          const auto* buf_1324 = buf[0];
 
           for(auto f1=0, f1324=0; f1!=n1; ++f1) {
             const auto bf1 = f1 + bf1_first;
@@ -647,19 +674,25 @@ Matrix compute_2body_fock_simple(const std::vector<Shell>& shells,
   return G;
 }
 
-Matrix compute_2body_fock(const std::vector<Shell>& shells,
+Matrix compute_2body_fock(const std::vector<libint2::Shell>& shells,
                           const Matrix& D) {
 
-  std::chrono::duration<double> time_elapsed = std::chrono::duration<double>::zero();
+  using libint2::Shell;
+  using libint2::Engine;
+  using libint2::Operator;
+
+  auto time_elapsed = std::chrono::duration<double>::zero();
 
   const auto n = nbasis(shells);
   Matrix G = Matrix::Zero(n,n);
 
   // construct the 2-electron repulsion integrals engine
-  libint2::TwoBodyEngine<libint2::Coulomb> engine(max_nprim(shells), max_l(shells), 0);
+  Engine engine(Operator::coulomb, max_nprim(shells), max_l(shells), 0);
 
   auto shell2bf = map_shell_to_basis_function(shells);
 
+  const auto& buf = engine.results();
+
   // The problem with the simple Fock builder is that permutational symmetries of the Fock,
   // density, and two-electron integrals are not taken into account to reduce the cost.
   // To make the simple Fock builder efficient we must rearrange our computation.
@@ -716,7 +749,10 @@ Matrix compute_2body_fock(const std::vector<Shell>& shells,
 
           const auto tstart = std::chrono::high_resolution_clock::now();
 
-          const auto* buf = engine.compute(shells[s1], shells[s2], shells[s3], shells[s4]);
+          engine.compute(shells[s1], shells[s2], shells[s3], shells[s4]);
+          const auto* buf_1234 = buf[0];
+          if (buf_1234 == nullptr)
+            continue; // if all integrals screened out, skip to next quartet
 
           const auto tstop = std::chrono::high_resolution_clock::now();
           time_elapsed += tstop - tstart;
@@ -741,7 +777,7 @@ Matrix compute_2body_fock(const std::vector<Shell>& shells,
                 for(auto f4=0; f4!=n4; ++f4, ++f1234) {
                   const auto bf4 = f4 + bf4_first;
 
-                  const auto value = buf[f1234];
+                  const auto value = buf_1234[f1234];
 
                   const auto value_scal_by_deg = value * s1234_deg;
 
